#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <error.h>
#include <errno.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>

#define max(A,B) ((A) >= (B) ? (A) : (B))
extern const char *__progname;

int
main(int argc, const char **argv) {
	int sock = -1;
	const char *socket_path;
	const char *message;

	struct sockaddr_un *server_address;
        size_t addr_size;
        socklen_t size;

	if (argc < 3) {
		fprintf(stderr, "Usage: %s <socket> <message>\n", __progname);
		return EXIT_SUCCESS;
	}
	socket_path = argv[1];
	message = argv[2];

	if (access(socket_path, W_OK) == -1)
		error(EXIT_FAILURE, errno, "access");

	if ((sock = socket(PF_UNIX, SOCK_STREAM, 0)) == -1)
		error(EXIT_FAILURE, errno, "socket");

	addr_size = sizeof (struct sockaddr_un) + max (0, strlen (socket_path) + 1 - (sizeof server_address->sun_path));

	server_address = (struct sockaddr_un *) malloc (addr_size);
	if (!server_address)
		error(EXIT_FAILURE, errno, "malloc");

	memset (server_address, 0, addr_size);

	server_address->sun_family = AF_UNIX;
	memcpy (server_address->sun_path, socket_path, strlen (socket_path));
	size = SUN_LEN (server_address);

	if (connect(sock, (struct sockaddr*)server_address, size) == -1)
		error(EXIT_FAILURE, errno, "connect");

	if (send(sock, message, strlen(message)+1, 0) == -1)
		error(EXIT_FAILURE, errno, "send");

	return EXIT_SUCCESS;
}
